<?php
/**
 * Concept from: Galileo Computing 2010
 * Stefan Reimers and Gunnar Thies: PHP 5.3 & MySQL 5.5
 * ISBN 978-3-8362-1645-6
 * 
 * Private error handler
 */

require_once PROJECT_DOCUMENT_ROOT . '/config.php';

/** 
 * Script for overwrite PHP Error-Handler
 */
class Error_Handling {

    public function __construct() {
            // overwrite php error handler with own error handler
            $old_error_handler = set_error_handler(array ($this, "userErrorHandler"));
    }

    /**
     * Error Handling
     *
     * Error will be dispayed or written to the log file.
     *
     * @param integer Error number
     * @param string Error message
     * @param string Filename
     * @param integer Line number
     */
    public function userErrorHandler($errno, $errmsg, $filename, $linenum) 
    {
        #global $use3;
        
        // Make assoc array with different errors
        $errortypes = array (E_ERROR => "Error", E_WARNING => "Warning",
            E_PARSE => "Parsing Error", E_NOTICE => "Notice", E_CORE_ERROR => "Core Error",
            E_CORE_WARNING => "Core Warning", E_COMPILE_ERROR => "Compile Error",
            E_COMPILE_WARNING => "Compile Warning", E_USER_ERROR => "User Error",
            E_USER_WARNING => "User Warning", E_USER_NOTICE => "User Notice",
            E_STRICT => "Runtime Notice");

        //Nur ausgeben, wenn der Fehler auch angezeigt werden soll
        if ((in_array($errno, $GLOBALS['screenErrors'])) && (EH_SCREEN_NOTIFICATION === true)) {
            //Fehlermeldung generieren
            echo '<fieldset style="border:1px solid red; padding:2px; margin:2px;">';
            echo '<legend style="color:red;font-family:Verdana;">';
            echo $errortypes[$errno]."</legend>";
            echo $errmsg."<br />\n";
            echo "<b>Skript:</b>&nbsp;".$filename."&nbsp;\n";
            echo '<b>Line:</b>&nbsp;<span style="color:green;font-weight:bold;">'.$linenum.
            "</span><br />\n";
            echo "</fieldset>";
        }

        //Fehler, die in der Datenbank bzw. der Log-Datei gespeichert werden sollen.
        if ((in_array($errno, $logErrors = $GLOBALS['logErrors'])) && (EH_LOG === true)) {
            //Routine zum Speichern des Fehlers in Datenbank (bzw. der Log-Datei)
            $this->doLog($errortypes[$errno], $errmsg, $filename, $linenum);
        }

        // Fehler, die per Email-Benachrichtigung an den Admin geschickt werden.
        if ((in_array($errno, $GLOBALS['emailErrors'])) && (EH_EMAIL_NOTIFICATION === true)) {
            //Fehlermeldung generieren
            $error = "Folgender Fehler ist aufgetreten:\r\n";
            $error .= $errortypes[$errno]."\r\n";
            $error .= $errmsg."\r\n";
            $error .= "In Skript: ".$filename."\r\n";
            $error .= "Zeile: ".$linenum."\r\n\r\n";
            $error .= "Diese Email wurde automatisch generiert.\n\n";
            
            $error_mail = "\nSERVER\n";
            foreach ($_SERVER as $key => $value) {
                $error_mail .= $key . '=> ' . $value . "\n";
            }
            
            $error_mail .= "\nSESSION\n";
            foreach ($_SESSION as $key => $value) {
                if ($key != 'faktura') {
                    $error_mail .= $key . ' => ' . $value . "\n";
                }
            }

            mail(EH_ADMIN_EMAIL, "Fehlermeldung", $error. $error_mail, 
                    "From: webmaster@".VERSION . VERSION_NO.".de\r\n");
        }

    }

    /**
     * Eintragen des aufgetretenen Fehlers in die Log-Datei
     *
     * @param integer Fehlernummer
     * @param string Fehlermeldung
     * @param string Dateiname
     * @param integer Zeilennummer
     */
    private function doLog($errortype, $errormsg, $filename, $linenum) {
        global $use;
        global $error_first_line;
        

        if(isset($_SESSION['use_id'])) {
            $use_id = $_SESSION['use_id'];
        }else{
            $use_id = 9999;
        }

       # if (isset ($db_log)) { // original
        if (LOCAL_SYSTEM == false) {
            $error = $errortype . ":" . $errormsg;
            $db_log = new Dbc();
            $sql = "INSERT INTO " . EH_DB_TABLE . " (user, error, script, line, timestamp)"
                 . " VALUES ('".$use_id."','".$error."', "."'"
                 . $filename . "'," . "'" . $linenum . "','" . time() . "')";
            $db_log->do_sql($sql);
        }
            //else{ // original

                $handle = fopen(EH_LOGFILE_PATH, 'a');
                //Fehlermedlung eintragen
                $error = $error_first_line . $errormsg . " | " . $filename 
                       . " row: " . $linenum . ' ' . $_SERVER['SCRIPT_NAME'] . "\r\n";

                fwrite($handle, $error);

                fclose($handle);
    }

}

//?berschreiben des "normalen" ErrorHandlings durch
//erstellen einer Instanz dieser Klasse
new Error_Handling();
